@@ -1,29 +1,21 @@
 # eg.
 # e=f.find('linktext','Download')
 # e.click()
-# e.send(KEYS.COMMAND,KEYS.SHIFT,KEYS.ARROW_LEFT)
-# e.send(KEYS.COMMAND,'a')
-# e.submit(), e.sendsubmit()
-# f.findall(By.CLASS_NAME,'d2l-datetime-selector-date-input.d2l-edit')  
+# e.send_keys(Keys.COMMAND,'a')
+# e.submit()
+# if space in class name, convert to .
+# f.find_all(By.CLASS_NAME,'d2l-datetime-selector-date-input.d2l-edit')  
 # https://stackoverflow.com/questions/17000703/is-string-matches-supported-in-selenium-webdriver-2
 # https://stackoverflow.com/questions/22436789/xpath-ends-with-does-not-work
-# f.findall('xpath','//*[starts-with(@id,"z_")]')  #contains(), but matches() ends-with() not supported by webdriver/xpath1.0
-# f.findall('xpath','//*[contains(@id,"time")][contains(@title,"Date")]')
-# f.findsel('id','activity--history',timeout=10).select_by_visible_text('history')
-#                                               .select_by_value()
-#                                               .select_by_index(2)  # 0-based
-#                                               .options[1].text  # first option's text
-# select = f.findsel('id','period_1_id',timeout=10)
-# options = select.options
-# for i in range(0,len(options)):
-#     option = options[i]
-#     if 'In Progress' in option.text:
-#         f.findsel('id','period_1_id',timeout=10).select_by_index(i)
-# f.save_screenshot()                                        | f.snap()
+# f.find_all('xpath','//*[starts-with(@id,"z_")]')  #contains(), but matches() ends-with() not supported by webdriver/xpath1.0
+# f.find_all('xpath','//*[contains(@id,"time")][contains(@title,"Date")]')
+# f.findsel('id','activity--history-range-dropdown',timeout=10).select_by_visible_text('history')
+#                                                              .select_by_index(2)  # 0-based
+# f.snap()
 # f.js_click(e)                                              | f.click(e)
 # f.js_rightclick(e)                                         | f.rclick(e)
-# f.js_setvalue(e,value)                                     | 
-# f.js_setvalue_via_typing(e,value)  # essentially send keys | f.send(e,value)
+# f.js_setvalue(e,value)                                     |
+# f.js_setvalue_via_typing(e,value)  # essentially send keys | f.send_keys(e,value)
 # f.execute_script("arguments[0].click();", e)
 # f.execute_script(f"arguments[0].value='{smscode}';", f.find('id','code'))
 # f.driver is the native webdriver object
@@ -46,7 +38,7 @@
 
 # System
 from typing import Optional, Union, List, Dict, Callable, Tuple
-import pickle, os, time, json, inspect, platform, tempfile, re
+import pickle, os, time, json, inspect, platform, tempfile
 
 # Pip
 from selenium.webdriver.support.ui import Select
@@ -56,7 +48,6 @@ from selenium.webdriver.common.keys import Keys
 from selenium.webdriver.support import expected_conditions as EC
 from selenium.webdriver.common.by import By as by
 from selenium.common.exceptions import NoSuchElementException
-from selenium.common.exceptions import TimeoutException
 from selenium.webdriver.firefox.options import Options as FirefoxOptions
 from selenium.webdriver.common.action_chains import ActionChains
 from selenium.webdriver.firefox.firefox_binary import FirefoxBinary
@@ -86,7 +77,7 @@ class Firefox:
 
     def __init__(
         self,
-        url = 'https://bot.sannysoft.com/',
+        url = None,
         headless: bool = False,
         wire: bool = False, # Use seleniumwire only when necessary, because it would slow things down a bit (not working if session is True)
         session: bool = False, # Use seleniumrequests for cross-session cookies (overrides wire)
@@ -95,7 +86,7 @@ class Firefox:
         service_log_path: Optional[str] = os.devnull,
         log_path: Optional[str] = os.devnull,
         cookies_folder_path: Optional[str] = None, # not working? Use session instead
-        extensions_folder_path: Optional[str] = None, # about:support  ~/Library/Application Support/Firefox/Profiles/h7d5u192.default-esr/extensions
+        extensions_folder_path: Optional[str] = None, # ~/Library/Application Support/Firefox/Profiles/h7d5u192.default-esr/extensions
         host: Optional[str] = None,
         port: Optional[int] = None,
         cookies_id: Optional[str] = None,
@@ -106,13 +97,12 @@ class Firefox:
         private: bool = False,
         screen_size: Optional[Tuple[int, int]] = None, # (width, height)
         full_screen: bool = False,
-        max_window: bool = True,  # overrides full_screen
         language: str = 'en-us',
         manual_set_timezone: bool = False,
         user_agent: Optional[str] = None,  # random
         load_proxy_checker_website: bool = False,
         disable_images: bool = False,
-        auto_close: bool = True, # no auto_close for keyboard maestro
+        autoclose: bool = True, # no autoclose for keyboard maestro
         *args, **kwargs
     ):
         '''EITHER PROVIDE 'cookies_id' OR  'cookies_folder_path'.
@@ -127,7 +117,7 @@ class Firefox:
            appropriate amount of time for the page or a part of page to load; so there is a module named
            expected_conditions.
         '''
-        self.auto_close=auto_close  # for destructor
+        self.autoclose=autoclose  # for destructor
         # force headless on linux
         if platform.system()=='Linux':
             headless=True
@@ -262,7 +252,7 @@ class Firefox:
             from seleniumrequests import Firefox
             self.driver = Firefox(
                 firefox_profile=profile,
-                options=options,
+                firefox_options=options,
                 service_log_path=service_log_path,
                 log_path=log_path,
                 firefox_binary=ff_binary,
@@ -272,7 +262,7 @@ class Firefox:
             if not wire:
                 self.driver = webdriver.Firefox(
                     firefox_profile=profile,
-                    options=options,
+                    firefox_options=options,
                     service_log_path=service_log_path,
                     log_path=log_path,
                     firefox_binary=ff_binary,
@@ -284,7 +274,7 @@ class Firefox:
                 }
                 self.driver = webdriver.Firefox(
                     firefox_profile=profile,
-                    options=options,
+                    firefox_options=options,
                     seleniumwire_options=seleniumwire_options,
                     service_log_path=service_log_path,
                     log_path=log_path,
@@ -295,9 +285,6 @@ class Firefox:
         if full_screen:
             self.driver.fullscreen_window()
 
-        if max_window:
-            self.driver.maximize_window()
-
         if extensions_folder_path is not None:
             try:
                 change_timezone_id = None
@@ -355,20 +342,9 @@ class Firefox:
                     self.driver.switch_to.window(self.driver.window_handles[-1])
                     self.driver.close()
 
-        # https://akarin.dev/2022/02/15/disable-geckodriver-detection-with-addon/
-        # customized addon, xpi is zip
-        # can be found at about:addons
-        HERE=os.path.dirname(os.path.abspath(__file__))
-        self.driver.install_addon(os.path.join(HERE,'firefox.xpi'), temporary=True)
         if url is not None: self.driver.get(url)
 
     # -------------------------------------------------------- Public methods -------------------------------------------------------- #
-    def install(self,xpi,signed=True):
-        """
-        signed: the package has been signed or not
-        """
-        self.driver.install_addon(xpi, temporary=(not signed))
-
     def wait(self,seconds=None):
         if seconds is None: seconds=self.wait_ajax
         time.sleep(seconds)
@@ -496,29 +472,21 @@ class Firefox:
             self.driver.set_page_load_timeout(300)
             return False
 
-    def getreq(self,url,regexpat,timeout=300,raw=False):
+    def getreq(self,url:str,regexpat,timeout=300):
         """
-        url: if '', will search visited urls 
         regexpat: The pat attribute will be matched within the request URL. pat a regular expression
-        if timeout and/or no match, returns ''
-        else returns 
-            if not raw: request.url
-            else request (so that can check request.headers, request.response.headers, request.response.status_code etc)
         """
+        
         # https://pypi.org/project/selenium-wire/
-
-        if url=='':
-            # Access requests via the `requests` attribute
-            # for request in driver.requests:
-            # request.url, request.response.status_code, request.response.headers['Content-Type']
-            for request in self.driver.iter_requests():
-                if request.response:
-                    if re.search(regexpat,request.url):
-                        if not raw:
-                            return request.url
-                        else:
-                            return request
-            return ''
+        # Access requests via the `requests` attribute
+        # for request in driver.requests:
+        # for request in driver.iter_requests():
+        #     if request.response:
+        #         print(
+        #             request.url,
+        #             request.response.status_code,
+        #             request.response.headers['Content-Type']
+        #         )
 
         # delete previous requests to get fresh results
         del self.driver.requests
@@ -531,26 +499,8 @@ class Firefox:
         # Also note that since pat can be a regular expression, 
         # you must escape special characters such as question marks with a slash. 
         # A TimeoutException is raised if no match is found within the timeout period.
-        try:
-            request = self.driver.wait_for_request(regexpat,timeout)
-            if not raw:
-                return request.url
-            else:
-                return request
-        except TimeoutException: 
-            return ''
-
-    def getresp(self,url:str,regexpat,timeout=300,raw=False,*args,**kwargs):
-        """
-        catch matched url, then request the url with headers, returns json (raw=False) or raw resp (raw=True)
-        """
-        import requests
-        resp=self.getreq(url,regexpat,timeout=timeout,raw=True)
-        resp=requests.get(resp.url,headers=resp.headers,*args,**kwargs)
-        if not raw:
-            return resp.json()
-        else:
-            return resp
+        request = self.driver.wait_for_request(regexpat,timeout)
+        return request.url
 
     def refresh(self,seconds=5) -> None:
         self.driver.refresh()
@@ -743,63 +693,40 @@ class Firefox:
     def move_to_element(self, element: Optional[WebElement]) -> bool:
         if not element:
             print('move_to_element: None element passed')
+
             return False
+
         try:
             actions = ActionChains(self.driver)
             actions.move_to_element(element).perform()
+
             return True
         except Exception as e:
             print('move_to_element', e)
+
             return False
 
     def switch_to_default(self):
-        # default is the first frame (frame 0)?
         self.driver.switch_to.default_content()
         return True
 
     def switch_to_frame(self, iframe: Optional[WebElement]) -> bool:
         if not iframe:
             print('switch_to_frame: None frame passed')
+
             return False
+
         try:
             self.driver.switch_to.default_content()
             time.sleep(1)
             self.driver.switch_to.frame(iframe)
+
             return True
         except Exception as e:
             print('switch_to_frame', e)
+
             return False
 
-    def frames(self):
-        self.driver.switch_to.default_content()
-        time.sleep(1)
-        n = self.driver.execute_script('return window.length')
-        res = []
-        for i in range(0,n):
-            self.driver.switch_to.default_content()
-            time.sleep(1)
-            self.driver.switch_to.frame(i)
-            name = self.driver.execute_script('return window.name')
-            res.append({'i':i,'name':name})
-        self.driver.switch_to.default_content()
-        return res
-
-    def find_frame(self,by,key,*args,**kwargs):
-        # first frame is 0
-        self.driver.switch_to.default_content()
-        time.sleep(1)
-        n = self.driver.execute_script('return window.length')
-        for i in range(0,n):
-            self.driver.switch_to.default_content()
-            time.sleep(1)
-            self.driver.switch_to.frame(i)
-            if self.find(by,key,*args,**kwargs) is not None:
-                name = self.driver.execute_script('return window.name')
-                self.driver.switch_to.default_content()
-                return({'i':i,'name':name})
-        self.driver.switch_to.default_content()
-        return False
-
     # returns x, y, w, h, max_x, max_y
     def get_element_coordinates(self, element) -> Tuple[int, int, int, int, int, int]:
         location = element.location
@@ -881,15 +808,11 @@ class Firefox:
 
     # essentially send keys
     def js_setvalue_via_typing(self, element: WebElement, value) -> bool:
-        is_mac=platform.system()=='Darwin'
-        if is_mac: 
-            element.send_keys(KEYS.COMMAND,"a")
-        else:
-            element.send_keys(KEYS.CONTROL,"a")
+        element.send_keys(Keys.COMMAND,"a")
         time.sleep(0.5)
         element.send_keys(value)
     send_keys=js_setvalue_via_typing
-    send=js_setvalue_via_typing
+
 
     # for D2L, this methods would not save value!
     def js_setvalue(self, element: WebElement, value) -> bool:
@@ -910,9 +833,9 @@ class Firefox:
 
     def execute_script(self, script: str, element: Optional[WebElement] = None) -> bool:
         try:
-            return self.driver.execute_script(script, element) if element else self.driver.execute_script(script)
+            self.driver.execute_script(script, element) if element else self.driver.execute_script(script)
 
-            # return True
+            return True
         except Exception as e:
             print('{}: {} - {}'.format(inspect.stack()[2][3], script, e))
 
@@ -923,7 +846,6 @@ class Firefox:
 
         def _cleanpath(path,allow_unicode=False):
             import unicodedata, urllib.parse
-            from ez import splitpath, joinpath
 
             [pth, fname, ext] = splitpath(path)
             fname = urllib.parse.unquote_plus(fname)
@@ -1006,8 +928,8 @@ class Firefox:
     # --------------------------------------------------------- Destructor ----------------------------------------------------------- #
 
     def __del__(self):
-        # no auto_close for keyboard maestro
-        if self.auto_close: 
+        # no autoclose for keyboard maestro
+        if self.autoclose: 
             try:
                 # prevent double quit if already manually called quit
                 if os.path.exists(self.driver.profile.path):
@@ -1048,10 +970,10 @@ class Firefox:
             By.TAG_NAME: By.TAG_NAME,
         }
         by=bydict[by]
-        key = key.strip()
         if by == By.CLASS_NAME:
-            # if space in class name, convert to . Jerry
+            # replace space with ., which is required by xpath?? Jerry
             key = key.replace(" ",".")
+
         if element is None:
             element = self.driver
         elif by == By.XPATH and not key.startswith('.'):
@@ -1062,68 +984,7 @@ class Firefox:
             es = WebDriverWait(element, timeout).until(
                 find_func((by, key))
             )
-            # jerry: alias enhanced
-            # https://stackoverflow.com/a/54662690/2292993
-            from functools import partial
-            def _send(
-                self,
-                keys: str,
-                delay=[0.025,0.25],
-                n=None
-            ) -> None:
-                # keys could only be single str with many chars
-                # delay in seconds after each key press, 0, [0.025,0.25] <-random between
-                import random
-                if type(delay) not in [tuple,list]: delay=[delay,delay]
-                for key in keys:
-                    self.send_keys(key)
-                    time.sleep(random.uniform(delay[0],delay[1]))
-            def _sendsubmit(self,*value,n=None):
-                # send keys and then submit
-                self.send(*value)
-                time.sleep(0.5)
-                self.submit()
-            def _getarea(self):
-                # https://stackoverflow.com/a/59347207/2292993
-                # Assume there is equal amount of browser chrome on the left and right sides of the screen.
-                canvas_x_offset = self.parent.execute_script("return window.screenX + (window.outerWidth - window.innerWidth) / 2 - window.scrollX;")
-                # Assume all the browser chrome is on the top of the screen and none on the bottom.
-                canvas_y_offset = self.parent.execute_script("return window.screenY + (window.outerHeight - window.innerHeight) - window.scrollY;")
-                # Get the element center.
-                element_area = (self.rect["x"] + canvas_x_offset,
-                                self.rect["y"] + canvas_y_offset,
-                                self.rect["width"],
-                                self.rect["height"])
-                return element_area
-            def _moveclick(self,radius=5,n=2,wait=0.25,*args,**kwargs):
-                """
-                n could be 0; 2 in case the first click only gets the focus
-                """
-                # # self is the element, self.parent is driver
-                # actions = ActionChains(self.parent)
-                # actions.move_to_element(self).click().perform()
-                import ez
-                ez.moveclick(self.getarea(), radius, n, wait, *args, **kwargs)
-            def _movesend(self,keys,delay=[0.025,0.25],radius=5,n=2,wait=0.25):
-                self.moveclick(radius,n,wait)
-                import time
-                time.sleep(wait)
-                self.send(keys,delay)
-            # es.send=es.send_keys
-            # finall
-            if find_func is EC.presence_of_all_elements_located: 
-                for e in es:
-                    e.send=partial(_send, e)
-                    e.sendsubmit=partial(_sendsubmit, e)
-                    e.getarea=partial(_getarea, e)
-                    e.moveclick=partial(_moveclick, e)
-                    e.movesend=partial(_movesend, e)
-            else:
-                es.send=partial(_send, es)
-                es.sendsubmit=partial(_sendsubmit, es)
-                es.getarea=partial(_getarea, es)
-                es.moveclick=partial(_moveclick, es)
-                es.movesend=partial(_movesend, es)
+
             return es
         except:
             return None
